@@ -1,12 +1,12 @@
-/*
- *  $Id: 6251568fe58fba1cd06f0a27d758c0f3b75f563e $
+/**
+ * \file defFind.c
  *
- *  Time-stamp:        "2009-11-08 08:14:31 bkorb"
+ *  Time-stamp:        "2010-07-11 12:45:27 bkorb"
  *
  *  This module locates definitions.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2009 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -43,26 +43,25 @@ static int hash_table_ct = 0;
 
 static char zDefinitionName[ AG_PATH_MAX ];
 
-static tDefEntry* findEntryByIndex( tDefEntry* pE, char* pzScan );
+static tDefEntry* findEntryByIndex(tDefEntry* pE, char* pzScan);
 
 #define ILLFORMEDNAME() \
-    AG_ABEND( aprf( zNameRef, zDefinitionName, \
-              pCurTemplate->pzTplFile, pCurMacro->lineNo ));
+    AG_ABEND(aprf(zNameRef, zDefinitionName, \
+              pCurTemplate->pzTplFile, pCurMacro->lineNo));
 
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by mk-fwd */
 static tDefEntry*
-findEntryByIndex( tDefEntry* pE, char* pzScan );
+findEntryByIndex(tDefEntry* pE, char* pzScan);
 
 static void
-addResult( tDefEntry* pDE, tDefEntryList* pDEL );
+addResult(tDefEntry* pDE, tDefEntryList* pDEL);
 
 static size_t
-badName( char* pzD, char const* pzS, size_t srcLen );
+badName(char* pzD, char const* pzS, size_t srcLen);
 
 static tDefEntry*
-defEntrySearch( char* pzName, tDefCtx* pDefCtx, ag_bool* pIsIndexed );
+find_def(char * pzName, tDefCtx* pDefCtx, ag_bool* pIsIndexed);
 
 static int
 hash_string(unsigned char const * pz);
@@ -71,11 +70,11 @@ static void
 stringAdd(char const * pz);
 
 static tDefEntry**
-entryListSearch( char* pzName, tDefCtx* pDefCtx );
+entryListSearch(char* pzName, tDefCtx* pDefCtx);
 /* = = = END-STATIC-FORWARD = = = */
 
 static tDefEntry*
-findEntryByIndex( tDefEntry* pE, char* pzScan )
+findEntryByIndex(tDefEntry* pE, char* pzScan)
 {
     int  idx;
 
@@ -104,7 +103,7 @@ findEntryByIndex( tDefEntry* pE, char* pzScan )
      */
     if (IS_DEC_DIGIT_CHAR(*pzScan)) {
         char* pz;
-        idx = strtol( pzScan, &pz, 0 );
+        idx = strtol(pzScan, &pz, 0);
 
         /*
          *  Skip over any trailing space and make sure we have a closer
@@ -133,7 +132,7 @@ findEntryByIndex( tDefEntry* pE, char* pzScan )
         {
             char  svch = *pzScan;
             *pzScan = NUL;
-            pzVal   = getDefine( pzDef, AG_TRUE );
+            pzVal   = getDefine(pzDef, AG_TRUE);
             *pzScan = svch;
         }
 
@@ -150,7 +149,7 @@ findEntryByIndex( tDefEntry* pE, char* pzScan )
         if ((pzVal == NULL) || (*pzVal == NUL))
             return NULL;
 
-        idx = strtol( pzVal, &pzDef, 0 );
+        idx = strtol(pzVal, &pzDef, 0);
 
         /*
          *  Make sure we got a legal number
@@ -181,13 +180,13 @@ findEntryByIndex( tDefEntry* pE, char* pzScan )
  *              list of found definitions (reallocating list size as needed).
  */
 static void
-addResult( tDefEntry* pDE, tDefEntryList* pDEL )
+addResult(tDefEntry* pDE, tDefEntryList* pDEL)
 {
     if (++(pDEL->usedCt) > pDEL->allocCt) {
         pDEL->allocCt += pDEL->allocCt + 8; /* 8, 24, 56, ... */
         pDEL->papDefEntry = (tDefEntry**)
-            AGREALOC( (void*)pDEL->papDefEntry, pDEL->allocCt * sizeof( void* ),
-                      "added find result" );
+            AGREALOC((void*)pDEL->papDefEntry, pDEL->allocCt * sizeof(void*),
+                     "added find result");
     }
 
     pDEL->papDefEntry[ pDEL->usedCt-1 ] = pDE;
@@ -195,12 +194,12 @@ addResult( tDefEntry* pDE, tDefEntryList* pDEL )
 
 
 static size_t
-badName( char* pzD, char const* pzS, size_t srcLen )
+badName(char* pzD, char const* pzS, size_t srcLen)
 {
-    memcpy( (void*)pzD, (void*)pzS, srcLen );
-    pzD[ srcLen ] = NUL;
-    fprintf( pfTrace, zNameRef, pzD,
-             pCurTemplate->pzTplFile, pCurMacro->lineNo );
+    memcpy((void*)pzD, (void*)pzS, srcLen);
+    pzD[srcLen] = NUL;
+    fprintf(pfTrace, zNameRef, pzD,
+             pCurTemplate->pzTplFile, pCurMacro->lineNo);
     return srcLen + 1;
 }
 
@@ -216,7 +215,7 @@ badName( char* pzD, char const* pzS, size_t srcLen )
  *  We start in CN_START.
  */
 LOCAL int
-canonicalizeName( char* pzD, char const* pzS, int srcLen )
+canonicalizeName(char* pzD, char const* pzS, int srcLen)
 {
     typedef enum {
         CN_START_NAME = 0,   /* must find a name */
@@ -266,7 +265,7 @@ canonicalizeName( char* pzD, char const* pzS, int srcLen )
     switch (state) {
     case CN_START_NAME:
         if (! IS_VAR_FIRST_CHAR(*pzS))
-            return badName( pzDst, pzOri, stLen );
+            return badName(pzDst, pzOri, stLen);
         state = CN_NAME_ENDED;  /* we found the start of our first name */
         break;  /* fall through to name/number consumption code */
 
@@ -289,7 +288,7 @@ canonicalizeName( char* pzD, char const* pzS, int srcLen )
         }
 
         if (--srcLen <= 0)
-            return badName( pzDst, pzOri, stLen );
+            return badName(pzDst, pzOri, stLen);
         goto nextSegment;
 
     case CN_INDEX:
@@ -320,7 +319,7 @@ canonicalizeName( char* pzD, char const* pzS, int srcLen )
          */
         if (*pzS == '$') {
             if (--srcLen < 0)
-                return badName( pzDst, pzOri, stLen );
+                return badName(pzDst, pzOri, stLen);
 
             *(pzD++) = *(pzS++);
             goto nextSegment;
@@ -332,7 +331,7 @@ canonicalizeName( char* pzD, char const* pzS, int srcLen )
          *  Nothing else is okay.
          */
         if ((*(pzD++) = *(pzS++)) != ']')
-            return badName( pzDst, pzOri, stLen );
+            return badName(pzDst, pzOri, stLen);
 
         if (--srcLen <= 0) {
             *pzD = NUL;
@@ -358,7 +357,7 @@ canonicalizeName( char* pzD, char const* pzS, int srcLen )
      *  Whatever, the next token must be a name or a number.
      */
     assert((state == CN_NAME_ENDED) || (state == CN_INDEX_CLOSE));
-    assert( IS_ALPHANUMERIC_CHAR(*pzS));
+    assert(IS_ALPHANUMERIC_CHAR(*pzS));
 
     /*
      *  Copy the name/number.  We already know the first character is valid.
@@ -367,9 +366,9 @@ canonicalizeName( char* pzD, char const* pzS, int srcLen )
     while (IS_VALUE_NAME_CHAR(*pzS)) {
         char ch = *(pzS++);
         if ((state != CN_INDEX_CLOSE) && IS_UPPER_CASE_CHAR(ch))
-            *(pzD++) = tolower( ch );
+            *(pzD++) = tolower(ch);
 
-        else switch ( ch ) { /* force the separator chars to be '_' */
+        else switch (ch) { /* force the separator chars to be '_' */
         case '-':
         case '^':
             *(pzD++) = '_';
@@ -389,9 +388,7 @@ canonicalizeName( char* pzD, char const* pzS, int srcLen )
 }
 
 
-/*
- *  findDefEntry
- *
+/**
  *  Find the definition entry for the name passed in.
  *  It is okay to find block entries IFF they are found on the
  *  current level.  Once you start traversing up the tree,
@@ -400,7 +397,7 @@ canonicalizeName( char* pzD, char const* pzS, int srcLen )
  *  to traverse the list of twins).
  */
 static tDefEntry*
-defEntrySearch( char* pzName, tDefCtx* pDefCtx, ag_bool* pIsIndexed )
+find_def(char * pzName, tDefCtx* pDefCtx, ag_bool* pIsIndexed)
 {
     char*        pcBrace;
     char         breakCh;
@@ -417,7 +414,7 @@ defEntrySearch( char* pzName, tDefCtx* pDefCtx, ag_bool* pIsIndexed )
      *  an index yet).
      */
     if (nestingDepth == 0) {
-        canonicalizeName( zDefinitionName, pzName, (int)strlen( pzName ));
+        canonicalizeName(zDefinitionName, pzName, (int)strlen(pzName));
         pzName = zDefinitionName;
 
         if (pIsIndexed != NULL)
@@ -430,7 +427,7 @@ defEntrySearch( char* pzName, tDefCtx* pDefCtx, ag_bool* pIsIndexed )
         }
     }
 
-    pcBrace  = pzName + strcspn( pzName, "[." );
+    pcBrace  = pzName + strcspn(pzName, "[.");
     breakCh  = *pcBrace;
     *pcBrace = NUL;
 
@@ -450,7 +447,7 @@ defEntrySearch( char* pzName, tDefCtx* pDefCtx, ag_bool* pIsIndexed )
              *  IF the name matches
              *  THEN break out of the double loop
              */
-            if (strcmp( pE->pzDefName, pzName ) == 0)
+            if (strcmp(pE->pzDefName, pzName) == 0)
                 goto found_def_entry;
 
             pE = pE->pNext;
@@ -488,14 +485,14 @@ defEntrySearch( char* pzName, tDefCtx* pDefCtx, ag_bool* pIsIndexed )
          */
         while (IS_WHITESPACE_CHAR(*++pcBrace )) ;
 
-        pE = findEntryByIndex( pE, pcBrace );
+        pE = findEntryByIndex(pE, pcBrace);
         if (pE == NULL)
             return pE;
 
         /*
          *  We must find the closing brace, or there is an error
          */
-        pcBrace = strchr( pcBrace, ']' );
+        pcBrace = strchr(pcBrace, ']');
         if (pcBrace == NULL)
             ILLFORMEDNAME();
 
@@ -548,7 +545,7 @@ defEntrySearch( char* pzName, tDefCtx* pDefCtx, ag_bool* pIsIndexed )
         for (;;) {
             tDefEntry* res;
 
-            res = defEntrySearch( pzName, &ctx, pIsIndexed );
+            res = find_def(pzName, &ctx, pIsIndexed);
             if ((res != NULL) || (breakCh == '[')) {
                 nestingDepth--;
                 return res;
@@ -661,11 +658,11 @@ stringAdd(char const * pz)
 }
 
 LOCAL tDefEntry*
-findDefEntry( char* pzName, ag_bool* pIsIndexed )
+findDefEntry(char * pzName, ag_bool* pIsIndexed)
 {
     if (HAVE_OPT(USED_DEFINES))
         stringAdd(pzName);
-    return defEntrySearch( pzName, &currDefCtx, pIsIndexed );
+    return find_def(pzName, &currDefCtx, pIsIndexed);
 }
 
 
@@ -708,7 +705,7 @@ print_used_defines(void)
  *  not try to traverse the list of twins).
  */
 static tDefEntry**
-entryListSearch( char* pzName, tDefCtx* pDefCtx )
+entryListSearch(char* pzName, tDefCtx* pDefCtx)
 {
     static tDefEntryList defList = { 0, 0, NULL, 0 };
 
@@ -736,12 +733,12 @@ entryListSearch( char* pzName, tDefCtx* pDefCtx )
             return NULL;
         }
 
-        canonicalizeName( zDefinitionName, pzName, (int)strlen( pzName ));
+        canonicalizeName(zDefinitionName, pzName, (int)strlen(pzName));
         pzName = zDefinitionName;
         defList.usedCt = 0;
     }
 
-    pcBrace  = pzName + strcspn( pzName, "[." );
+    pcBrace  = pzName + strcspn(pzName, "[.");
     breakCh  = *pcBrace;
     *pcBrace = NUL;
 
@@ -771,7 +768,7 @@ entryListSearch( char* pzName, tDefCtx* pDefCtx )
              *  IF the name matches
              *  THEN go add it, plus all its twins
              */
-            if (strcmp( pE->pzDefName, pzName ) == 0)
+            if (strcmp(pE->pzDefName, pzName) == 0)
                 goto found_def_entry;
 
             pE = pE->pNext;
@@ -802,7 +799,7 @@ entryListSearch( char* pzName, tDefCtx* pDefCtx )
     switch (breakCh) {
     case NUL:
         do  {
-            addResult( pE, &defList );
+            addResult(pE, &defList);
             pE = pE->pTwin;
         } while (pE != NULL);
         goto returnResult;
@@ -813,14 +810,14 @@ entryListSearch( char* pzName, tDefCtx* pDefCtx )
          */
         while (IS_WHITESPACE_CHAR(*++pcBrace)) ;
 
-        pE = findEntryByIndex( pE, pcBrace );
+        pE = findEntryByIndex(pE, pcBrace);
         if (pE == NULL)
             goto returnResult;
 
         /*
          *  We must find the closing brace, or there is an error
          */
-        pcBrace = strchr( pcBrace, ']' );
+        pcBrace = strchr(pcBrace, ']');
         if (pcBrace == NULL)
             ILLFORMEDNAME();
 
@@ -854,45 +851,37 @@ entryListSearch( char* pzName, tDefCtx* pDefCtx )
     }
 
     /*
-     *  We cannot find a member of a non-block type macro definition.
-     */
-    if (pE->valType != VALTYP_BLOCK)
-        return NULL;
-
-    /*
-     *  Loop through all the twins of the entry.  We ignore twins if we just
-     *  used a subscript.
+     *  Loop through all the twins of the entry.  We only look once if we used
+     *  a subscript.  Ignore any entry types that are not "Blocks" because
+     *  text entries won't have any children.
      */
     defList.nestLevel++;
-    {
-        tDefCtx ctx = { NULL, &currDefCtx };
+    do  {
+        tDefCtx ctx = { pE->val.pDefEntry, &currDefCtx };
 
-        ctx.pDefs = pE->val.pDefEntry;
+        if (pE->valType == VALTYP_BLOCK)
+            (void)entryListSearch(pzName, &ctx);
+
+        if (breakCh == '[')
+            break;
+
+        pE = pE->pTwin;
+    } while (pE != NULL);
 
-        for (;;) {
-            (void)entryListSearch( pzName, &ctx );
-            if (breakCh == '[')
-                break;
-            pE = pE->pTwin;
-            if (pE == NULL)
-                break;
-            ctx.pDefs = pE->val.pDefEntry;
-        }
-    }
     defList.nestLevel--;
 
  returnResult:
     if (defList.nestLevel == 0)
-        addResult( NULL, &defList );
+        addResult(NULL, &defList);
 
     return defList.papDefEntry;
 }
 
 
 LOCAL tDefEntry**
-findEntryList( char* pzName )
+findEntryList(char* pzName)
 {
-    return entryListSearch( pzName, &currDefCtx );
+    return entryListSearch(pzName, &currDefCtx);
 }
 /*
  * Local Variables:
