@@ -1,7 +1,10 @@
-/*-
+
+/**
+ * \file fmemopen.c
+ *
  * Copyright (c) 2004-2010 by Bruce Korb.  All rights reserved.
  *
- * Time-stamp:      "2010-02-24 08:42:29 bkorb"
+ * Time-stamp:      "2010-07-10 16:33:59 bkorb"
  *
  * This code was inspired from software written by
  *   Hanno Mueller, kontakt@hanno.de
@@ -114,7 +117,7 @@ typedef unsigned char buf_bytes_t;
 typedef struct fmem_cookie_s fmem_cookie_t;
 struct fmem_cookie_s {
     mode_bits_t    mode;
-    buf_bytes_t   *buffer;
+    buf_bytes_t *  buffer;
     size_t         buf_size;    /* Full size of buffer */
     size_t         next_ix;     /* Current position */
     size_t         eof;         /* End Of File */
@@ -125,33 +128,39 @@ struct fmem_cookie_s {
 
 /* = = = START-STATIC-FORWARD = = = */
 static int
-fmem_getmode( char const *pMode, mode_bits_t *pRes );
+fmem_getmode(char const *pMode, mode_bits_t *pRes);
 
 static int
-fmem_extend( fmem_cookie_t *pFMC, size_t new_size );
+fmem_extend(fmem_cookie_t *pFMC, size_t new_size);
 
 static ssize_t
-fmem_read( void *cookie, void *pBuf, size_t sz );
+fmem_read(void *cookie, void *pBuf, size_t sz);
 
 static ssize_t
-fmem_write( void *cookie, const void *pBuf, size_t sz );
+fmem_write(void *cookie, const void *pBuf, size_t sz);
 
 static seek_pos_t
-fmem_seek( void *cookie, fmem_off_t *p_offset, int dir);
+fmem_seek(void * cookie, fmem_off_t * p_offset, int dir);
 
 static int
-fmem_close( void *cookie );
-/* = = = END-STATIC-FORWARD = = = */
+fmem_close(void * cookie);
 
-FILE * ag_fmemopen(void *buf, ssize_t len, char const *pMode);
-int fmem_ioctl( FILE* fp, int req, void* ptr );
+static ag_bool
+fmem_config_user_buf(fmem_cookie_t * pFMC, void * buf, ssize_t len);
+
+static ag_bool
+fmem_alloc_buf(fmem_cookie_t * pFMC, ssize_t len);
+/* = = = END-STATIC-FORWARD = = = */
 
 #ifdef TEST_FMEMOPEN
   static fmem_cookie_t* saved_cookie = NULL;
 #endif
 
+/**
+ * Convert a mode string into mode bits.
+ */
 static int
-fmem_getmode( char const *pMode, mode_bits_t *pRes )
+fmem_getmode(char const *pMode, mode_bits_t *pRes)
 {
     if (pMode == NULL)
         return 1;
@@ -186,8 +195,11 @@ fmem_getmode( char const *pMode, mode_bits_t *pRes )
     return 0;
 }
 
+/**
+ * Extend the space associated with an fmem file.
+ */
 static int
-fmem_extend( fmem_cookie_t *pFMC, size_t new_size )
+fmem_extend(fmem_cookie_t *pFMC, size_t new_size)
 {
     size_t ns = (new_size + (pFMC->pg_size - 1)) & (~(pFMC->pg_size - 1));
 
@@ -202,16 +214,16 @@ fmem_extend( fmem_cookie_t *pFMC, size_t new_size )
          *  Previously, this was a user supplied buffer.  We now move to one
          *  of our own.  The user is responsible for the earlier memory.
          */
-        void* bf = malloc( ns );
+        void* bf = malloc(ns);
         if (bf == NULL)
             goto no_space;
 
-        memcpy( bf, pFMC->buffer, pFMC->buf_size );
+        memcpy(bf, pFMC->buffer, pFMC->buf_size);
         pFMC->buffer = bf;
         pFMC->mode  |= FLAG_BIT(allocated);
     }
     else {
-        void* bf = realloc( pFMC->buffer, ns );
+        void* bf = realloc(pFMC->buffer, ns);
         if (bf == NULL)
             goto no_space;
 
@@ -221,7 +233,7 @@ fmem_extend( fmem_cookie_t *pFMC, size_t new_size )
     /*
      *  Unallocated file space is set to zeros.  Emulate that.
      */
-    memset( pFMC->buffer + pFMC->buf_size, 0, ns - pFMC->buf_size );
+    memset(pFMC->buffer + pFMC->buf_size, 0, ns - pFMC->buf_size);
     pFMC->buf_size = ns;
     return 0;
 
@@ -230,8 +242,11 @@ fmem_extend( fmem_cookie_t *pFMC, size_t new_size )
     return -1;
 }
 
+/**
+ * Handle file system callback to read data from our string.
+ */
 static ssize_t
-fmem_read( void *cookie, void *pBuf, size_t sz )
+fmem_read(void *cookie, void *pBuf, size_t sz)
 {
     fmem_cookie_t *pFMC = cookie;
 
@@ -248,9 +263,11 @@ fmem_read( void *cookie, void *pBuf, size_t sz )
     return sz;
 }
 
-
+/**
+ * Handle file system callback to write data to our string
+ */
 static ssize_t
-fmem_write( void *cookie, const void *pBuf, size_t sz )
+fmem_write(void *cookie, const void *pBuf, size_t sz)
 {
     fmem_cookie_t *pFMC = cookie;
     int add_nul_char;
@@ -276,7 +293,7 @@ fmem_write( void *cookie, const void *pBuf, size_t sz )
     {
         size_t next_pos = pFMC->next_ix + sz + add_nul_char;
         if (next_pos > pFMC->buf_size) {
-            if (fmem_extend( pFMC, next_pos ) != 0) {
+            if (fmem_extend(pFMC, next_pos) != 0) {
                 /*
                  *  We could not extend the memory.  Try to write some data.
                  *  Fail if we are either at the end or not writing data.
@@ -294,7 +311,7 @@ fmem_write( void *cookie, const void *pBuf, size_t sz )
         }
     }
 
-    memcpy( pFMC->buffer + pFMC->next_ix, pBuf, sz);
+    memcpy(pFMC->buffer + pFMC->next_ix, pBuf, sz);
 
     pFMC->next_ix += sz;
 
@@ -315,9 +332,11 @@ fmem_write( void *cookie, const void *pBuf, size_t sz )
     return sz;
 }
 
-
+/**
+ * Handle file system callback to set a new current position
+ */
 static seek_pos_t
-fmem_seek( void *cookie, fmem_off_t *p_offset, int dir)
+fmem_seek(void * cookie, fmem_off_t * p_offset, int dir)
 {
     size_t new_pos;
     fmem_cookie_t *pFMC = cookie;
@@ -335,11 +354,12 @@ fmem_seek( void *cookie, fmem_off_t *p_offset, int dir)
         goto seek_oops;
 
     if (new_pos > pFMC->buf_size) {
-        if (fmem_extend( pFMC, new_pos ))
+        if (fmem_extend(pFMC, new_pos))
             return -1; /* errno is set */
     }
 
     pFMC->next_ix = new_pos;
+    *p_offset     = (fmem_off_t)new_pos;
     return new_pos;
 
  seek_oops:
@@ -347,75 +367,110 @@ fmem_seek( void *cookie, fmem_off_t *p_offset, int dir)
     return -1;
 }
 
-
+/**
+ * Free up the memory associated with an fmem file.
+ * If the user is managing the space, then the allocated bit is set.
+ */
 static int
-fmem_close( void *cookie )
+fmem_close(void * cookie)
 {
-    fmem_cookie_t *pFMC = cookie;
+    fmem_cookie_t * pFMC = cookie;
 
     if (pFMC->mode & FLAG_BIT(allocated))
-        free( pFMC->buffer );
-    free( pFMC );
+        free(pFMC->buffer);
+    free(pFMC);
 
     return 0;
 }
 
-/*=export_func fmem_ioctl
- *
- *  what:  get information about a string stream
- *
- *  arg: + FILE* + fptr  + the string stream
- *  arg: + int   + req   + the requested data
- *  arg: + void* + ptr   + ptr to result area
- *
- *  ret-type:  int
- *  ret-desc:  zero on success
- *
- *  err: non-zero is returned and @code{errno} is set to @code{EINVAL}.
- *
- *  doc:
- *
- *  This routine surreptitiously slips in a special request.
- *  The commands supported are:
- *
- *  @table @code
- *  @item FMEM_IOCTL_BUF_ADDR
- *
- *    Retrieve the address of the buffer.  Future output to the stream might
- *    cause this buffer to be freed and the contents copied to another buffer.
- *    You must ensure that either you have saved the buffer (see
- *    @code{FMEM_IOCTL_SAVE_BUF} below), or do not do any more I/O to it while
- *    you are using this address.
- *
- *    "ptr" must point to a @code{char*} pointer.
- *
- *  @item FMEM_IOCTL_SAVE_BUF
- *
- *    Do not deallocate the buffer on close.  You would likely want to use
- *    this after writing all the output data and just before closing.
- *    Otherwise, the buffer might get relocated.  Once you have specified
- *    this, the current buffer becomes the client program's resposibility to
- *    @code{free()}.  If more I/O operations are performed, a new buffer
- *    @i{may} get allocated.  @code{fmem_close} will free that new buffer
- *    and the user will remain responsible for @code{free()}-ing this buffer.
- *
- *    "ptr" must point to a @code{char*} pointer.
- *
- *  @end table
- *
- *  The third argument is never optional and must be a pointer to where data
- *  are to be retrieved or stored.  It may be NULL if there are no data to
- *  transfer, but both of these functions currently return the address of the
- *  buffer.  This is implemented as a wrapper around @code{fseek(3C)}, so
- *  the "req" argument must not conflict with @code{SEEK_SET}, @code{SEEK_CUR}
- *  or @code{SEEK_END}.
-=*/
-int
-fmem_ioctl( FILE* fp, int req, void* ptr )
+/**
+ * Configure the user supplied buffer.
+ */
+static ag_bool
+fmem_config_user_buf(fmem_cookie_t * pFMC, void * buf, ssize_t len)
 {
-    if (fseek( fp, (long)ptr, req ) < 0)
-        return -1;
-    return 0;
+    /*
+     *  User allocated buffer.  User responsible for disposal.
+     */
+    if (len == 0) {
+        free(pFMC);
+        errno = EINVAL;
+        return AG_FALSE;
+    }
+
+    pFMC->buffer = (buf_bytes_t*)buf;
+
+    /*  Figure out where our "next byte" and EOF are.
+     *  Truncated files start at the beginning.
+     */
+    if (pFMC->mode & FLAG_BIT(truncate)) {
+        /*
+         *  "write" mode
+         */
+        pFMC->eof = \
+            pFMC->next_ix = 0;
+    }
+
+    else if (pFMC->mode & FLAG_BIT(binary)) {
+        pFMC->eof = len;
+        pFMC->next_ix    = (pFMC->mode & FLAG_BIT(append)) ? len : 0;
+
+    } else {
+        /*
+         * append or read text mode -- find the end of the buffer
+         * (the first NUL character)
+         */
+        buf_bytes_t *p = (buf_bytes_t*)buf;
+
+        pFMC->eof = 0;
+        while ((*p != NUL) && (++(pFMC->eof) < len))  p++;
+        pFMC->next_ix =
+            (pFMC->mode & FLAG_BIT(append)) ? pFMC->eof : 0;
+    }
+
+    /*
+     *  text mode - NUL terminate buffer, if it fits.
+     */
+    if (  ((pFMC->mode & FLAG_BIT(binary)) == 0)
+       && (pFMC->next_ix < len)) {
+        pFMC->buffer[pFMC->next_ix] = NUL;
+    }
+
+    pFMC->buf_size = len;
+    return AG_TRUE;
+}
+
+/**
+ * Allocate an initial buffer for fmem.
+ */
+static ag_bool
+fmem_alloc_buf(fmem_cookie_t * pFMC, ssize_t len)
+{
+    /*
+     *  We must allocate the buffer.  If "len" is zero, set it to page size.
+     */
+    pFMC->mode |= FLAG_BIT(allocated);
+    if (len == 0)
+        len = pFMC->pg_size;
+
+    /*
+     *  Unallocated file space is set to NULs.  Emulate that.
+     */
+    pFMC->buffer = calloc((size_t)1, (size_t)len);
+    if (pFMC->buffer == NULL) {
+        errno = ENOMEM;
+        free(pFMC);
+        return AG_FALSE;
+    }
+
+    /*
+     *  We've allocated the buffer.  The end of file and next entry
+     *  are both zero.
+     */
+    pFMC->next_ix  = 0;
+    pFMC->eof      = 0;
+    pFMC->buf_size = len;
+    return AG_TRUE;
 }
 
 /*=export_func ag_fmemopen
@@ -525,88 +580,21 @@ ag_fmemopen(void *buf, ssize_t len, char const *pMode)
         pFMC->pg_size = getpagesize();
 
     if (buf != NULL) {
-        /*
-         *  User allocated buffer.  User responsible for disposal.
-         */
-        if (len == 0) {
-            free( pFMC );
-            errno = EINVAL;
+        if (! fmem_config_user_buf(pFMC, buf, len))
             return NULL;
-        }
 
-        /*  Figure out where our "next byte" and EOF are.
-         *  Truncated files start at the beginning.
-         */
-        if (pFMC->mode & FLAG_BIT(truncate)) {
-            /*
-             *  "write" mode
-             */
-            pFMC->eof = \
-            pFMC->next_ix = 0;
-        }
-
-        else if (pFMC->mode & FLAG_BIT(binary)) {
-            pFMC->eof = len;
-            pFMC->next_ix    = (pFMC->mode & FLAG_BIT(append)) ? len : 0;
-
-        } else {
-            /*
-             * append or read text mode -- find the end of the buffer
-             * (the first NUL character)
-             */
-            buf_bytes_t *p = pFMC->buffer = (buf_bytes_t*)buf;
-            pFMC->eof = 0;
-            while ((*p != NUL) && (++(pFMC->eof) < len))  p++;
-            pFMC->next_ix =
-                (pFMC->mode & FLAG_BIT(append)) ? pFMC->eof : 0;
-        }
-
-        /*
-         *  text mode - NUL terminate buffer, if it fits.
-         */
-        if (  ((pFMC->mode & FLAG_BIT(binary)) == 0)
-           && (pFMC->next_ix < len)) {
-            pFMC->buffer[pFMC->next_ix] = NUL;
-        }
-    }
-
-    else if ((pFMC->mode & (FLAG_BIT(append) | FLAG_BIT(truncate))) == 0) {
+    } else if ((pFMC->mode & (FLAG_BIT(append) | FLAG_BIT(truncate))) == 0) {
         /*
          *  Not appending and not truncating.  We must be reading.
          *  We also have no user supplied buffer.  Nonsense.
          */
         errno = EINVAL;
-        free( pFMC );
+        free(pFMC);
         return NULL;
     }
 
-    else {
-        /*
-         *  We must allocate the buffer.  If "len" is zero, set it to page size.
-         */
-        pFMC->mode |= FLAG_BIT(allocated);
-        if (len == 0)
-            len = pFMC->pg_size;
-
-        /*
-         *  Unallocated file space is set to NULs.  Emulate that.
-         */
-        pFMC->buffer = calloc((size_t)1, (size_t)len);
-        if (pFMC->buffer == NULL) {
-            errno = ENOMEM;
-            free( pFMC );
-            return NULL;
-        }
-
-        /*
-         *  We've allocated the buffer.  The end of file and next entry
-         *  are both zero.
-         */
-        pFMC->next_ix = 0;
-        pFMC->eof = 0;
-    }
-
-    pFMC->buf_size   = len;
+    else if (! fmem_alloc_buf(pFMC, len))
+        return NULL;
 
 #ifdef TEST_FMEMOPEN
     saved_cookie = pFMC;
@@ -617,6 +605,9 @@ ag_fmemopen(void *buf, ssize_t len, char const *pMode)
             ?  (cookie_read_function_t*)fmem_read  : NULL;
         cookie_write_function_t* pWr = (pFMC->mode & FLAG_BIT(write))
             ? (cookie_write_function_t*)fmem_write : NULL;
+
+        FILE * res;
+
 #if defined(HAVE_FOPENCOOKIE)
         cookie_io_functions_t iof;
         iof.read  = pRd;
@@ -624,14 +615,14 @@ ag_fmemopen(void *buf, ssize_t len, char const *pMode)
         iof.seek  = (cookie_seek_function_t* )fmem_seek;
         iof.close = (cookie_close_function_t*)fmem_close;
 
-        return fopencookie( pFMC, pMode, iof );
-#elif defined(HAVE_FUNOPEN)
-        return funopen( pFMC, pRd, pWr,
-                        (cookie_seek_function_t* )fmem_seek,
-                        (cookie_close_function_t*)fmem_close );
+        res = fopencookie(pFMC, pMode, iof);
 #else
-#       include "We have neither fopencookie(3GNU) nor funopen(3BSD)"
+        res = funopen(pFMC, pRd, pWr,
+                      (cookie_seek_function_t* )fmem_seek,
+                      (cookie_close_function_t*)fmem_close);
 #endif
+
+        return res;
     }
 }
 
