@@ -1,7 +1,8 @@
 
-/*
- *  $Id: putshell.c,v 4.26 2009/01/01 16:51:49 bkorb Exp $
- * Time-stamp:      "2008-07-27 12:14:38 bkorb"
+/**
+ * \file putshell.c
+ *
+ * Time-stamp:      "2010-07-17 10:31:37 bkorb"
  *
  *  This module will interpret the options set in the tOptions
  *  structure and print them to standard out in a fashion that
@@ -9,7 +10,7 @@
  *
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is copyright (c) 1992-2009 by Bruce Korb - all rights reserved
+ *  AutoOpts is Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -23,15 +24,29 @@
  *
  *  These files have the following md5sums:
  *
- *  239588c55c22c60ffe159946a760a33e pkg/libopts/COPYING.gplv3
- *  fa82ca978890795162346e661b47161a pkg/libopts/COPYING.lgplv3
+ *  43b91e8ca915626ed3818ffb1b71248b pkg/libopts/COPYING.gplv3
+ *  06a1a2e4760c90ea5e1dad8dfaac4d39 pkg/libopts/COPYING.lgplv3
  *  66a5cedaf62c4b2637025f049f9b826f pkg/libopts/COPYING.mbsd
  */
+static char const zOptValFmt[] = "%s_%s=";
+static char const zOptEnd[]    = "\nexport %s_%s\n";
+static char const zOptNumFmt[] = "%1$s_%2$s=%3$d # 0x%3$X\nexport %1$s_%2$s\n";
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by mk-fwd */
 static void
-putQuotedStr( tCC* pzStr );
+print_quot_str(tCC* pzStr);
+
+static void
+print_enumeration(tOptions * pOpts, tOptDesc * pOD);
+
+static void
+print_membership(tOptions * pOpts, tOptDesc * pOD);
+
+static void
+print_stacked_arg(tOptions * pOpts, tOptDesc * pOD);
+
+static void
+print_reordering(tOptions * pOpts);
 /* = = = END-STATIC-FORWARD = = = */
 
 /*
@@ -39,13 +54,13 @@ putQuotedStr( tCC* pzStr );
  *  been emitted and the closing quote will be upon return.
  */
 static void
-putQuotedStr( tCC* pzStr )
+print_quot_str(tCC* pzStr)
 {
     /*
      *  Handle empty strings to make the rest of the logic simpler.
      */
     if ((pzStr == NULL) || (*pzStr == NUL)) {
-        fputs( "''", stdout );
+        fputs("''", stdout);
         return;
     }
 
@@ -54,7 +69,7 @@ putQuotedStr( tCC* pzStr )
      *  bail if that is all we need to do.
      */
     while (*pzStr == '\'') {
-        fputs( "\\'", stdout );
+        fputs("\\'", stdout);
         pzStr++;
     }
     if (*pzStr == NUL)
@@ -63,38 +78,144 @@ putQuotedStr( tCC* pzStr )
     /*
      *  Start the single quote string
      */
-    fputc( '\'', stdout );
+    fputc('\'', stdout);
     for (;;) {
-        tCC* pz = strchr( pzStr, '\'' );
+        tCC* pz = strchr(pzStr, '\'');
         if (pz == NULL)
             break;
 
         /*
          *  Emit the string up to the single quote (apostrophe) we just found.
          */
-        (void)fwrite( pzStr, (size_t)(pz - pzStr), (size_t)1, stdout );
-        fputc( '\'', stdout );
+        (void)fwrite(pzStr, (size_t)(pz - pzStr), (size_t)1, stdout);
+        fputc('\'', stdout);
         pzStr = pz;
 
         /*
          *  Emit an escaped apostrophe for every one we find.
          *  If that ends the string, do not re-open the single quotes.
          */
-        while (*++pzStr == '\'')   fputs( "\\'", stdout );
+        while (*++pzStr == '\'')   fputs("\\'", stdout);
         if (*pzStr == NUL)
             return;
 
-        fputc( '\'', stdout );
+        fputc('\'', stdout);
     }
 
     /*
      *  If we broke out of the loop, we must still emit the remaining text
      *  and then close the single quote string.
      */
-    fputs( pzStr, stdout );
-    fputc( '\'', stdout );
+    fputs(pzStr, stdout);
+    fputc('\'', stdout);
 }
 
+static void
+print_enumeration(tOptions * pOpts, tOptDesc * pOD)
+{
+    uintptr_t e_val = pOD->optArg.argEnum;
+    printf(zOptValFmt, pOpts->pzPROGNAME, pOD->pz_NAME);
+
+    /*
+     *  Convert value to string, print that and restore numeric value.
+     */
+    (*(pOD->pOptProc))(OPTPROC_RETURN_VALNAME, pOD);
+    printf("'%s'", pOD->optArg.argString);
+    if (pOD->fOptState & OPTST_ALLOC_ARG)
+        AGFREE(pOD->optArg.argString);
+    pOD->optArg.argEnum = e_val;
+
+    printf(zOptEnd, pOpts->pzPROGNAME, pOD->pz_NAME);
+}
+
+static void
+print_membership(tOptions * pOpts, tOptDesc * pOD)
+{
+    char const * pz;
+    uintptr_t val = 1;
+    printf(zOptNumFmt, pOpts->pzPROGNAME, pOD->pz_NAME,
+           (int)(uintptr_t)(pOD->optCookie));
+    pOD->optCookie = (void*)(uintptr_t)~0UL;
+    (*(pOD->pOptProc))(OPTPROC_RETURN_VALNAME, pOD);
+
+    /*
+     *  We are building the typeset list.  The list returned starts with
+     *  'none + ' for use by option saving stuff.  We must ignore that.
+     */
+    pz = pOD->optArg.argString + 7;
+    while (*pz != NUL) {
+        printf("typeset -x -i %s_", pOD->pz_NAME);
+        while (IS_PLUS_N_SPACE_CHAR(*pz))  pz++;
+
+        for (;;) {
+            int ch = *(pz++);
+            if (IS_LOWER_CASE_CHAR(ch))   fputc(toupper(ch), stdout);
+            else if (IS_UPPER_CASE_CHAR(ch))   fputc(ch, stdout);
+            else if (IS_PLUS_N_SPACE_CHAR(ch)) goto name_done;
+            else if (ch == NUL)        { pz--; goto name_done; }
+            else fputc('_', stdout);
+        } name_done:;
+        printf("=%1$lu # 0x%1$lX\n", (unsigned long)val);
+        val <<= 1;
+    }
+
+    AGFREE(pOD->optArg.argString);
+    pOD->optArg.argString = NULL;
+    pOD->fOptState &= ~OPTST_ALLOC_ARG;
+}
+
+static void
+print_stacked_arg(tOptions * pOpts, tOptDesc * pOD)
+{
+    tSCC zOptCookieCt[] = "%1$s_%2$s_CT=%3$d\nexport %1$s_%2$s_CT\n";
+
+    tArgList*    pAL = (tArgList*)pOD->optCookie;
+    tCC**        ppz = pAL->apzArgs;
+    int          ct  = pAL->useCt;
+
+    printf(zOptCookieCt, pOpts->pzPROGNAME, pOD->pz_NAME, ct);
+
+    while (--ct >= 0) {
+        tSCC numarg_z[] = "%s_%s_%d=";
+        tSCC end_z[]    = "\nexport %s_%s_%d\n";
+
+        printf(numarg_z, pOpts->pzPROGNAME, pOD->pz_NAME,
+               pAL->useCt - ct);
+        print_quot_str(*(ppz++));
+        printf(end_z, pOpts->pzPROGNAME, pOD->pz_NAME,
+               pAL->useCt - ct);
+    }
+}
+
+static void
+print_reordering(tOptions * pOpts)
+{
+    int  optIx;
+
+    fputs("set --", stdout);
+
+    for (optIx = pOpts->curOptIdx; optIx < pOpts->origArgCt; optIx++) {
+
+        char* pzArg = pOpts->origArgVect[ optIx ];
+
+        if (strchr(pzArg, '\'') == NULL)
+            printf(" '%s'", pzArg);
+
+        else {
+            fputs(" '", stdout);
+            for (;;) {
+                char ch = *(pzArg++);
+                switch (ch) {
+                case '\'':  fputs("'\\''", stdout); break;
+                case NUL:   goto arg_done;
+                default:    fputc(ch, stdout); break;
+                }
+            } arg_done:;
+            fputc('\'', stdout);
+        }
+    }
+    fputs("\nOPTION_CT=0\n", stdout);
+}
 
 /*=export_func  optionPutShell
  * what:  write a portable shell script to parse options
@@ -104,18 +225,15 @@ putQuotedStr( tCC* pzStr )
  *        the options described in the option definitions.
 =*/
 void
-optionPutShell( tOptions* pOpts )
+optionPutShell(tOptions* pOpts)
 {
     int  optIx = 0;
     tSCC zOptCtFmt[]  = "OPTION_CT=%d\nexport OPTION_CT\n";
-    tSCC zOptNumFmt[] = "%1$s_%2$s=%3$d # 0x%3$X\nexport %1$s_%2$s\n";
     tSCC zOptDisabl[] = "%1$s_%2$s=%3$s\nexport %1$s_%2$s\n";
-    tSCC zOptValFmt[] = "%s_%s=";
-    tSCC zOptEnd[]    = "\nexport %s_%s\n";
     tSCC zFullOptFmt[]= "%1$s_%2$s='%3$s'\nexport %1$s_%2$s\n";
     tSCC zEquivMode[] = "%1$s_%2$s_MODE='%3$s'\nexport %1$s_%2$s_MODE\n";
 
-    printf( zOptCtFmt, pOpts->curOptIdx-1 );
+    printf(zOptCtFmt, pOpts->curOptIdx-1);
 
     do  {
         tOptDesc* pOD = pOpts->pOptDesc + optIx;
@@ -143,7 +261,7 @@ optionPutShell( tOptions* pOpts )
             p->optArg     = pOD->optArg;
             p->fOptState &= OPTST_PERSISTENT_MASK;
             p->fOptState |= pOD->fOptState & ~OPTST_PERSISTENT_MASK;
-            printf( zEquivMode, pOpts->pzPROGNAME, pOD->pz_NAME, p->pz_NAME );
+            printf(zEquivMode, pOpts->pzPROGNAME, pOD->pz_NAME, p->pz_NAME);
             pOD = p;
         }
 
@@ -153,37 +271,7 @@ optionPutShell( tOptions* pOpts )
          *  of bitmask value and we need to emit the bit values.
          */
         if (OPTST_GET_ARGTYPE(pOD->fOptState) == OPARG_TYPE_MEMBERSHIP) {
-            char const * pz;
-            uintptr_t val = 1;
-            printf( zOptNumFmt, pOpts->pzPROGNAME, pOD->pz_NAME,
-                    (int)(uintptr_t)(pOD->optCookie) );
-            pOD->optCookie = (void*)(uintptr_t)~0UL;
-            (*(pOD->pOptProc))(OPTPROC_RETURN_VALNAME, pOD);
-
-            /*
-             *  We are building the typeset list.  The list returned starts with
-             *  'none + ' for use by option saving stuff.  We must ignore that.
-             */
-            pz = pOD->optArg.argString + 7;
-            while (*pz != NUL) {
-                printf( "typeset -x -i %s_", pOD->pz_NAME );
-                while (IS_PLUS_N_SPACE_CHAR(*pz))  pz++;
-
-                for (;;) {
-                  int ch = *(pz++);
-                       if (IS_LOWER_CASE_CHAR(ch))   fputc(toupper(ch), stdout);
-                  else if (IS_UPPER_CASE_CHAR(ch))   fputc(ch, stdout);
-                  else if (IS_PLUS_N_SPACE_CHAR(ch)) goto name_done;
-                  else if (ch == NUL)        { pz--; goto name_done; }
-                  else fputc( '_', stdout );
-                } name_done:;
-                printf( "=%1$lu # 0x%1$lX\n", (unsigned long)val );
-                val <<= 1;
-            }
-
-            AGFREE(pOD->optArg.argString);
-            pOD->optArg.argString = NULL;
-            pOD->fOptState &= ~OPTST_ALLOC_ARG;
+            print_membership(pOpts, pOD);
             continue;
         }
 
@@ -193,122 +281,84 @@ optionPutShell( tOptions* pOpts )
          *  The idea is that if someone defines an option to initialize
          *  enabled, we should tell our shell script that it is enabled.
          */
-        if (UNUSED_OPT( pOD ) && DISABLED_OPT( pOD ))
+        if (UNUSED_OPT(pOD) && DISABLED_OPT(pOD)) {
             continue;
+        }
 
         /*
          *  Handle stacked arguments
          */
         if (  (pOD->fOptState & OPTST_STACKED)
            && (pOD->optCookie != NULL) )  {
-            tSCC zOptCookieCt[] = "%1$s_%2$s_CT=%3$d\nexport %1$s_%2$s_CT\n";
-
-            tArgList*    pAL = (tArgList*)pOD->optCookie;
-            tCC**        ppz = pAL->apzArgs;
-            int          ct  = pAL->useCt;
-
-            printf( zOptCookieCt, pOpts->pzPROGNAME, pOD->pz_NAME, ct );
-
-            while (--ct >= 0) {
-                tSCC numarg_z[] = "%s_%s_%d=";
-                tSCC end_z[]    = "\nexport %s_%s_%d\n";
-
-                printf( numarg_z, pOpts->pzPROGNAME, pOD->pz_NAME,
-                        pAL->useCt - ct );
-                putQuotedStr( *(ppz++) );
-                printf( end_z, pOpts->pzPROGNAME, pOD->pz_NAME,
-                        pAL->useCt - ct );
-            }
+            print_stacked_arg(pOpts, pOD);
+            continue;
         }
 
         /*
          *  If the argument has been disabled,
          *  Then set its value to the disablement string
          */
-        else if ((pOD->fOptState & OPTST_DISABLED) != 0)
-            printf( zOptDisabl, pOpts->pzPROGNAME, pOD->pz_NAME,
-                    (pOD->pz_DisablePfx != NULL)
-                    ? pOD->pz_DisablePfx : "false" );
+        if ((pOD->fOptState & OPTST_DISABLED) != 0) {
+            printf(zOptDisabl, pOpts->pzPROGNAME, pOD->pz_NAME,
+                   (pOD->pz_DisablePfx != NULL)
+                   ? pOD->pz_DisablePfx : "false");
+            continue;
+        }
 
         /*
          *  If the argument type is numeric, the last arg pointer
          *  is really the VALUE of the string that was pointed to.
          */
-        else if (OPTST_GET_ARGTYPE(pOD->fOptState) == OPARG_TYPE_NUMERIC)
-            printf( zOptNumFmt, pOpts->pzPROGNAME, pOD->pz_NAME,
-                    (int)pOD->optArg.argInt );
+        if (OPTST_GET_ARGTYPE(pOD->fOptState) == OPARG_TYPE_NUMERIC) {
+            printf(zOptNumFmt, pOpts->pzPROGNAME, pOD->pz_NAME,
+                   (int)pOD->optArg.argInt);
+            continue;
+        }
 
         /*
          *  If the argument type is an enumeration, then it is much
          *  like a text value, except we call the callback function
          *  to emit the value corresponding to the "optArg" number.
          */
-        else if (OPTST_GET_ARGTYPE(pOD->fOptState) == OPARG_TYPE_ENUMERATION) {
-            uintptr_t e_val = pOD->optArg.argEnum;
-            printf( zOptValFmt, pOpts->pzPROGNAME, pOD->pz_NAME );
-
-            /*
-             *  Convert value to string, print that and restore numeric value.
-             */
-            (*(pOD->pOptProc))(OPTPROC_RETURN_VALNAME, pOD);
-            printf("'%s'", pOD->optArg.argString);
-            if (pOD->fOptState & OPTST_ALLOC_ARG)
-                AGFREE(pOD->optArg.argString);
-            pOD->optArg.argEnum = e_val;
-
-            printf(zOptEnd, pOpts->pzPROGNAME, pOD->pz_NAME);
+        if (OPTST_GET_ARGTYPE(pOD->fOptState) == OPARG_TYPE_ENUMERATION) {
+            print_enumeration(pOpts, pOD);
+            continue;
         }
 
         /*
          *  If the argument type is numeric, the last arg pointer
          *  is really the VALUE of the string that was pointed to.
          */
-        else if (OPTST_GET_ARGTYPE(pOD->fOptState) == OPARG_TYPE_BOOLEAN)
-            printf( zFullOptFmt, pOpts->pzPROGNAME, pOD->pz_NAME,
-                    (pOD->optArg.argBool == 0) ? "false" : "true" );
+        if (OPTST_GET_ARGTYPE(pOD->fOptState) == OPARG_TYPE_BOOLEAN) {
+            printf(zFullOptFmt, pOpts->pzPROGNAME, pOD->pz_NAME,
+                   (pOD->optArg.argBool == 0) ? "false" : "true");
+            continue;
+        }
 
         /*
          *  IF the option has an empty value,
          *  THEN we set the argument to the occurrence count.
          */
-        else if (  (pOD->optArg.argString == NULL)
-                || (pOD->optArg.argString[0] == NUL) )
+        if (  (pOD->optArg.argString == NULL)
+           || (pOD->optArg.argString[0] == NUL) ) {
 
-            printf( zOptNumFmt, pOpts->pzPROGNAME, pOD->pz_NAME,
-                    pOD->optOccCt );
+            printf(zOptNumFmt, pOpts->pzPROGNAME, pOD->pz_NAME,
+                   pOD->optOccCt);
+            continue;
+        }
 
         /*
          *  This option has a text value
          */
-        else {
-            printf( zOptValFmt, pOpts->pzPROGNAME, pOD->pz_NAME );
-            putQuotedStr( pOD->optArg.argString );
-            printf( zOptEnd, pOpts->pzPROGNAME, pOD->pz_NAME );
-        }
+        printf(zOptValFmt, pOpts->pzPROGNAME, pOD->pz_NAME);
+        print_quot_str(pOD->optArg.argString);
+        printf(zOptEnd, pOpts->pzPROGNAME, pOD->pz_NAME);
+
     } while (++optIx < pOpts->presetOptCt );
 
     if (  ((pOpts->fOptSet & OPTPROC_REORDER) != 0)
-       && (pOpts->curOptIdx < pOpts->origArgCt)) {
-        fputs( "set --", stdout );
-        for (optIx = pOpts->curOptIdx; optIx < pOpts->origArgCt; optIx++) {
-            char* pzArg = pOpts->origArgVect[ optIx ];
-            if (strchr( pzArg, '\'' ) == NULL)
-                printf( " '%s'", pzArg );
-            else {
-                fputs( " '", stdout );
-                for (;;) {
-                    char ch = *(pzArg++);
-                    switch (ch) {
-                    case '\'':  fputs( "'\\''", stdout ); break;
-                    case NUL:   goto arg_done;
-                    default:    fputc( ch, stdout ); break;
-                    }
-                } arg_done:;
-                fputc( '\'', stdout );
-            }
-        }
-        fputs( "\nOPTION_CT=0\n", stdout );
-    }
+       && (pOpts->curOptIdx < pOpts->origArgCt))
+        print_reordering(pOpts);
 }
 
 /*
