@@ -1,11 +1,10 @@
-/*
- *  $Id: 6d52e6abaefa9e965f3e09dcf94c4efd98f4b44a $
+/**
+ * \file gdinit.c
  *
- *  getdefs copyright (c) 1999-2009 by Bruce Korb - all rights reserved
+ *  getdefs Copyright (c) 1999-2010 by Bruce Korb - all rights reserved
  *
  *  Author:            Bruce Korb <bkorb@gnu.org>
- *  Time-stamp:        "2009-07-22 19:04:30 bkorb"
- *            by: bkorb
+ *  Time-stamp:        "2010-07-17 09:48:00 bkorb"
  *
  *  This file is part of AutoGen.
  *  AutoGen copyright (c) 1992-2009 by Bruce Korb - all rights reserved
@@ -24,10 +23,9 @@
  *  with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-tSCC zNoList[] = "ERROR:  block attr must have name list:\n\t%s\n";
+static char const zNoList[] = "ERROR:  block attr must have name list:\n\t%s\n";
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by mk-fwd */
 static char*
 compressOptionText( char* pzS, char* pzE );
 
